# -*- coding: utf-8 -*-
"""
第211回農林交流センターWS「メッシュ農業気象データ利用講習会」 
実習：Pythonによるメッシュ農業気象データの処理２

aet17.pyの有効積算気温の計算を、そのまま面に拡張したもの。
指定した領域内の各メッシュグリッドごとに、aet17.pyの計算を繰り返します。
ただし、海の領域には無効値が入っていて計算できないので、その部分は飛ばすようになっています。
"""
#Pythonに機能を追加するために外部モジュールをインポートします。
import numpy as np               #配列計算を高速に実行するためのモジュール
import AMD_Tools3 as AMD         #メッシュ農業気象データを利用するためのモジュール

#---- ０）気象データを取得--------------

#ちょっと奇妙ですが、プログラムの本文は次の文で始めます。
if __name__ == "__main__":

    # 取得する気象要素の指定
    #  気象要素は、文字列で指定します。 
    nani = "TMP_mea"

    # データを取得する期間の指定
    #  2018年3月1日から7月31日までのデータを取得するには以下のように期間を指定します。
    itsu = ["2018-03-01", "2018-07-31" ]

    # 対象とする地点の指定
    # 北緯36.0566度,東経140.125度の地点を含むメッシュのデータを取得するには場所を以下のように指定します。
    doko = [ 35.8, 36.3, 139.7, 141]	#つくば周辺の矩形領域

    # 気象データの取得
    #　　気象データを取得するGetMetData関数の括弧の中に上記の変数を指定し、右辺に置きます。
    #　　左辺には関数の戻り値を置きます。順に、気象データ、日付、緯度、経度が格納されます。
    met, tim, lat, lon = AMD.GetMetData(nani, itsu, doko)	#気象データを取得
    #この場合のmetは本当に三次元なので、一次元への変換は行いません。
    T=met
    
    ntim = len(tim)  #何日分の気象データかを数えます。
    nlat = len(lat)  #南北方向（緯度方向）のメッシュ数を数えます。
    nlon = len(lon)  #東西方向（経度方向）のメッシュ数を数えます。

#-----１）有効積算気温の計算 -------------#
    Ts = 10.0       #基準温度
    Tccx = 950.0    #有効積算気温の上限値
    Tcc = np.zeros((ntim,nlat,nlon))   #有効積算気温の入れ物（三次元）
    
    #ここから各メッシュごとに計算
    for y in range(nlat):
        for x in range(nlon):
            Tyx = T[:,y,x]               #メッシュ[y,x]における気温の時系列データを取り出す
            for t in range(ntim):
                if np.isnan(Tyx[t]):     #メッシュの値が"nan"（無効値）の場合は海や湖。
                    Tcc[:,y,x] = np.nan  #すべての日の積算気温の入れ物に無効値を入れて、
                    break  #このメッシュについては日ごとの計算は放棄する。
                if t == 0:        # まず最初の日（t=0）を計算する。
                    Tcc[t,y,x] = max(Tyx[t]-Ts, 0.0)
                else:             #２日目以降は、「t-1日目までの積算温度」に「t日目の有効温度」を足す。
                    Tcc[t,y,x] = Tcc[t-1,y,x] + max(Tyx[t]-Ts, 0.0)

    #ここまでで全メッシュの計算が終了
    Tccend = Tcc[ntim-1,:,:]    #最終日の有効積算気温分布（二次元） ☆今度は上限値でのストップはありません

#------２）計算結果の地図表示（メッシュ図を.pngファイルに出力＋地理院地図に重ねて表示するhtmlを生成） -----#
    AMD.PutGSI_Map(Tccend,lat,lon,
                   label="Effective accumulated temperature[degC]",
                   cmapstr=None,minmax=None)
    """
引数(必須)：
    data：表示させるデータ（2次元numpyアレイ）
    lat：緯度（1次元numpyアレイ）
    lon：経度（1次元numpyアレイ）
引数(必要に応じ指定)：
    label：凡例のタイトルの文字列
    cmapstr：カラーマップを指定（詳細は後述）
    minmax：カラースケールを指定（[min,max]のリスト型）
    filename：出力ファイル名
    outdir:出力フォルダ名
カラーマップについて：
    カラーマップには名称があるのでこれを文字列で("で囲んで)指定する。
    例)　レインボーカラー:rainbow、黄色-オレンジ-赤の順で変化:YlOrRdなど
    色の順序をを反転させたい場合は、rainbow_rのよう名称の後ろに"_r"を付加する。
    詳細は下記URLを参照。
        http://matplotlib.org/examples/color/colormaps_reference.html
    """
#------３）計算結果のテキスト出力 -----#
    # AMD_Tools3 の中の PutCSV_MT 関数を使用する。
    # 3-1） 「最終日の有効積算気温」を書き出す---値が「２次元・非時系列」の場合
    AMD.PutCSV_MT(Tccend,lat,lon,
                  addlalo=True,header=None,filename='EAT-map.csv')
    # 3-2) 「期間中の日平均気温」を書き出す---値が「２次元・時系列」の場合
    AMD.PutCSV_MT(met,lat,lon,
                  addlalo=True,header=None,filename='Temp-map_TS.csv')

# 【PutCSV_MT の用法(抜粋）】　詳細はAMD_Tools3.py を参照してください
# Addlalo=True で、「緯度、経度」を出力（Falseだと３次メッシュコードのみ）
# Header=None で、行頭のヘッダをなし（必要であればヘッダを足します）
# Filename=“ファイル名” で出力ファイル名を指定

